from typing import Union

from vestapol.destinations.gcp_destination import GoogleCloudPlatform
from vestapol.destinations.local_destination import Local

DestinationTypes = Union[GoogleCloudPlatform, Local]



from __future__ import annotations

from abc import abstractmethod
from typing import Dict, List, Tuple, Any, Optional, TYPE_CHECKING

from vestapol.web_resources import base_resource
from vestapol.writers import json_writer

if TYPE_CHECKING:
    from vestapol.destinations import DestinationTypes


class JSONResource(base_resource.BaseResource):
    response_format_tag = "json"
    external_data_format_tag = "jsonl"
    response_filename = "data.json"

    def __init__(
        self,
        name,
        base_url,
        endpoint,
        version,
        query_params=None,
        request_headers=None,
        manual_schema=None,
    ):
        self.name: str = name
        self.base_url: str = base_url
        self.endpoint: str = endpoint
        self.version: str = version
        self.query_params: Optional[dict] = query_params
        self.request_headers: Optional[dict] = request_headers
        self.manual_schema: Optional[List[Dict]] = manual_schema
        super().__init__(
            self.name,
            self.base_url,
            self.endpoint,
            self.version,
            self.response_format_tag,
            self.external_data_format_tag,
            self.response_filename,
            self.query_params,
            self.request_headers,
            self.manual_schema,
        )

    def load(self, destination: DestinationTypes):
        data = self.request_data()
        self.write_data(data, destination)
        self.unnest_data(data, destination)
        return data

    def write_data(self, api_data, destination: DestinationTypes):
        json_writer.write_json(
            api_data, self.response_target_prefix / self.response_filename, destination
        )

    @abstractmethod
    def unnest_data(self, data, destination):
        pass

    def write_list(
        self,
        data: List[Dict],
        destination: DestinationTypes,
        data_path: List[Tuple[str, Any]] = None,
    ):
        if not isinstance(data, list):
            raise TypeError(f"Expected a list, received {type(data)}")

        pathname = self.get_pathname(data_path)
        json_writer.write_jsonl(data=data, pathname=pathname, destination=destination)

    def write_dict(
        self,
        data: Dict,
        destination: DestinationTypes,
        data_path: List[Tuple[str, Any]] = None,
    ):
        if not isinstance(data, dict):
            raise TypeError(f"Expected a dict, received {type(data)}")

        pathname = self.get_pathname(data_path)
        json_writer.write_json(data=data, pathname=pathname, destination=destination)



from typing import Union

from vestapol.web_resources.csv_resource import CSVResource
from vestapol.web_resources.github_resource import GitHubResource
from vestapol.web_resources.json_resource import JSONResource

ResourceTypes = Union[CSVResource, GitHubResource, JSONResource]



from __future__ import annotations

import pathlib
from abc import ABC
from abc import abstractmethod
from typing import List, Tuple, Any, Optional, TYPE_CHECKING

from pendulum import DateTime

from vestapol.api import api

if TYPE_CHECKING:
    from vestapol.destinations import DestinationTypes


class BaseResource(ABC):
    def __init__(
        self,
        name: str,
        base_url: Optional[str],
        endpoint: Optional[str],
        version: str,
        response_format_tag: str,
        external_data_format_tag: str,
        response_filename: str = "data",
        query_params: dict = None,
        request_headers: dict = None,
        manual_schema=None,
    ):
        self.name = name
        self.base_url = base_url
        self.endpoint = endpoint
        self.version = version
        self.response_format_tag = response_format_tag
        self.external_data_format_tag = external_data_format_tag
        self.response_filename = response_filename
        self.query_params = query_params
        self.request_headers = request_headers
        self.manual_schema = manual_schema
        self.requested_at = DateTime.utcnow().replace(microsecond=0)

    def load(self, destination: DestinationTypes):
        data = self.request_data()
        self.write_data(data, destination)
        return data

    def request_data(self):
        return api.get_api_data(
            f"{self.base_url}{self.endpoint}",
            self.response_format_tag,
            self.query_params,
            self.request_headers,
        )

    @abstractmethod
    def write_data(self, data, destination: DestinationTypes):
        pass

    def get_response_root(self, format_tag: str):
        return pathlib.Path(
            self.name, format_tag, self.version, self.requested_at_hive_path
        )

    @property
    def response_target_prefix(self):
        self._response_target_prefix = self.get_response_root(self.response_format_tag)
        return self._response_target_prefix

    @property
    def requested_at_hive_path(self):
        return f'requested_at={self.requested_at.strftime("%Y-%m-%d %H:%M:%S")}'

    @staticmethod
    def get_hive_path(data_path: List[Tuple[str, Any]]) -> str:

        prefix_components = []
        for x in data_path:
            prefix_components.append(f"{x[0]}={x[1]}")

        hive_path = "/".join(prefix_components)

        return hive_path

    def get_pathname(self, data_path: List[Tuple[str, Any]] = None) -> str:
        target_prefix = self.get_response_root(self.external_data_format_tag)

        if data_path:
            hive_path = self.get_hive_path(data_path)
            pathname = target_prefix / hive_path / self.response_filename
        else:
            pathname = target_prefix / self.response_filename

        return pathname
